﻿using Models.model;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace AQuarantineAdventure
{
    /// <summary>
    /// Lógica de interacción para Consequence_Window.xaml
    /// </summary>
    public partial class Consequence_Window : Window
    {
        Model myModel;
        Consequence cons;
        int cont_happy;
        Start_Window start;
        Consequence_Window cw;
     

        public Consequence_Window()
        {
            InitializeComponent();
            myModel = Model.getInstance();
            chekisTheEnd();

        }

        /// <summary>
        /// Checks if the actual location is the end of the game
        /// </summary>
        private void chekisTheEnd()
        {
            Locs loc = myModel.getCurrentLoc();
            if (loc.IdLoc.Equals("LOC_006"))
            {
                img_end.Visibility = Visibility.Visible;
                Rolechar rc = myModel.getRolechar();
                tb_cons.Text = rc.TxtFin.ToString();
                Game game = myModel.getGame();
                lb_totalhappy.Content = "Felicidad total " + game.Ghappiness;
                User current = myModel.getCurrentUser();
                int winCoins = game.Ghappiness / 100;
                current.Coins = current.Coins + winCoins;
                lb_happy.Content = "Monedas conseguidas: " + winCoins;
                //post of user
                actualizeUser(current);
            }
            else
            {
                getConsecuencesPetition();
                initTexts();
            }
        }

        // get the consecuence based on de id of the  chosen answer
        private async void getConsecuencesPetition()
        {
            Answer_option ans = new Answer_option();

            ans = myModel.getAnswer();
            try
            {
                string URL_consequence = Properties.Settings.Default.URL_consequence + ans.Consequence_code;
                Console.WriteLine(URL_consequence); // solo test          
                ResponseResult uResult = await myModel.getPetition(URL_consequence);
                if (uResult.ResultCode == 1)
                {
                    cons = myModel.convertDataToConsequence(uResult.Data);
                    myModel.setConsequence(cons);
                    checkGameOver(cons);
                }
                else if (uResult.ResultCode == 0)
                {
                    MessageBox.Show("Error ! Algo sucedió durante el proceso.", "Error!", MessageBoxButton.OK, MessageBoxImage.Error);
                }
            }
            catch (WebException)
            {
                MessageBox.Show("Error ! Error con la conexión.", "Error!", MessageBoxButton.OK, MessageBoxImage.Error);
                // error connection message
            }
            catch (JsonReaderException)
            {
                MessageBox.Show("Error ! Algo sucedió durante el proceso.", "Error!", MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        /// <summary>
        /// Checks if it is game over==true
        /// </summary>
        /// <param name="conse"> the consequence of game over</param>
        private void checkGameOver(Consequence conse)
        {
            if (conse.Game_over == true)
            {
                tb_cons.Text = conse.Cons_desc;
                img_gameover.Visibility = Visibility.Visible;
                btn_end.Visibility = Visibility.Visible;
                lb_happy.Visibility = Visibility.Hidden;
                lb_totalhappy.Visibility = Visibility.Hidden;
                btn_ok.Visibility = Visibility.Hidden;
            }
        }

        //initialize the labels and textblock
        private void initTexts()
        {
            lb_happy.Content = "Felicidad: " + cons.Reward + "+100";
            tb_cons.Text = cons.Cons_desc;
            sumReward();
        }

        // operations for count the puntuation
        private void sumReward()
        {
            Game game = myModel.getGame();
            cont_happy = (game.Ghappiness + cons.Reward) + 100;
            game.Ghappiness = cont_happy;
            lb_totalhappy.Content = "Felicidad total: " + game.Ghappiness;
        }

        // Hide the consequence window or return back if its the last location
        private void btn_ok_Click(object sender, RoutedEventArgs e)
        {           
            Locs loc = myModel.getCurrentLoc();
            if (loc.IdLoc.Equals("LOC_006"))
            {
                Game game = myModel.getGame();
                Max_score maxscore = myModel.getMaxScore();
                if (maxscore.Max_happiness < game.Ghappiness)
                {
                    // post del score ( update)
                    actualizeMaxScore();
                }
                start = new Start_Window();
                start.Visibility = Visibility.Visible;
                
            }
            this.Visibility = Visibility.Hidden;
        }

        /// <summary>
        /// Actualize the current user info in the db
        /// </summary>
        /// <param name="actuser"> the current user</param>
        private async void actualizeUser(User actuser)
        {
            string URL_actualizeUser = Properties.Settings.Default.URL_actualizeuser;
            Console.WriteLine(URL_actualizeUser);
            try
            {
                ResponseResult result = await myModel.deserializeActualizeUserResult(URL_actualizeUser, actuser);// transforms User  into a json format
                if (result.ResultCode == 0)
                {
                    MessageBox.Show("Uoops! Ocurrió un error durante el proceso", "Error!", MessageBoxButton.OK, MessageBoxImage.Error);
                  
                }
                else if (result.ResultCode == -1)
                {
                    MessageBox.Show("Vaya! Parece que el usuario no existe", "Error!", MessageBoxButton.OK, MessageBoxImage.Error);
                   
                }
                else if (result.ResultCode == 1)
                {
                    //OK
                   // MessageBox.Show("Se ha actualizado el usuario!", "Registro Actualizado", MessageBoxButton.OK, MessageBoxImage.Information);                                
                }
            }
            catch (WebException ex)
            {
                Console.WriteLine("error " + ex.Message);
                // mensaje de error de conexion)
            }
            catch (JsonReaderException jre)
            {
                Console.WriteLine("error " + jre.Message);
            }
        }
    
    /// <summary>
    /// Actualize the max score of the current user
    /// </summary>
    private async void actualizeMaxScore()
        {
            User user = myModel.getCurrentUser();
            Game game = myModel.getGame();
            string iduser = user.IdUser;
            int newscore = game.Ghappiness;
            Max_score maxscore = new Max_score(iduser, newscore);
            string URL_actualizescore = Properties.Settings.Default.URL_setScore;
            Console.WriteLine(URL_actualizescore);
            try
            {
                ResponseResult result = await myModel.deserializeResultScore(URL_actualizescore, maxscore);// transforms User  into a json format
                if (result.ResultCode == 1)
                {                
                    MessageBox.Show("Se ha actualizado la puntuacion maxima!", "Record", MessageBoxButton.OK, MessageBoxImage.Error);
                   
                }
                else if (result.ResultCode == 2)
                {
                    MessageBox.Show("Record máximo añadido!", "Record", MessageBoxButton.OK, MessageBoxImage.Information);
                                    
                }
                else if (result.ResultCode == 0)
                {
                    MessageBox.Show("Error con la conexión", "Error!", MessageBoxButton.OK, MessageBoxImage.Error);
                }
            }
            catch (WebException ex)
            {
                Console.WriteLine("error " + ex.Message);
                // mensaje de error de conexion)
            }
            catch (JsonReaderException jre)
            {
                Console.WriteLine("error " + jre.Message);
            }
        }
        
        // button end, opens the start window when the game is over
        private void Button_end_Click(object sender, RoutedEventArgs e)
        {
            start = new Start_Window();
            start.Show();
            this.Close();
        }
    }
}

