﻿using Models.model;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace AQuarantineAdventure
{
    /// <summary>
    /// Lógica de interacción para Event_Window.xaml
    /// </summary>
    public partial class Event_Window : Window
    {
        Consequence_Window cons;
        Model myModel;
        Event ev;     
        List<Answer_option> answers;
        Map_Window map;
        
        public Event_Window()
        {
            InitializeComponent();          
            initBackgrounds();
            myModel = Model.getInstance();      
            generateRandEvent();          
        }

        //-----------------EVENT--------------------------------

        /// <summary>
        /// Gets a random event from database, based on de actual LOC
        /// </summary>
        private void generateRandEvent()
        {

            //switch case para cada caso
            Locs loc = new Locs();
            loc = myModel.getCurrentLoc();
            ev = new Event();
            switch (loc.IdLoc)
            {
                case "LOC_000": // show start 
                    startLocation();                   
                    break;

                case "LOC_001":// show store
                    storeLocation();                   
                    break;

                case "LOC_002"://generate random event
                    getRandomEvent(loc.IdLoc);
                    img_park.Visibility = Visibility.Visible;                  
                    break;

                case "LOC_003"://generate random event
                    getRandomEvent(loc.IdLoc);
                    img_bridge.Visibility = Visibility.Visible;                   
                    break;

                case "LOC_004"://generate random event
                    getRandomEvent(loc.IdLoc);
                    img_forest.Visibility = Visibility.Visible;
                    break;

                case "LOC_005"://generate random event
                    getRandomEvent(loc.IdLoc);
                    img_square.Visibility = Visibility.Visible;
                    break;

                case "LOC_006": // show final                  
                    endLocation();
                    break;
                
            }
                      
        }

        /// <summary>
        /// Gets the random event from model, depending on the id of current location
        /// </summary>
        /// <param name="idloc"> the id of current location</param>
        private async void getRandomEvent(string idloc)
        {
            try
            {
                string URL_getEvent = Properties.Settings.Default.URL_randEvent + idloc;
                Console.WriteLine(URL_getEvent); // solo test          
                ResponseResult uResult = await myModel.getPetition(URL_getEvent);
                if (uResult.ResultCode == 1)
                {
                    ev = myModel.convertDataToEvent(uResult.Data);
                   // Console.WriteLine(ev.ToString());..
                    myModel.setCurrentEvent(ev);
                    tb_event.Text = ev.Event_text.ToString();                  
                    getPetitionAnswers(ev.IdEvent);                  
                }
                else if (uResult.ResultCode == 0)
                {
                    MessageBox.Show("Error ! El evento no existe", "Error!", MessageBoxButton.OK, MessageBoxImage.Error);
                }
            }
            catch (WebException ex)
            {
                Console.WriteLine("error " + ex.Message);
                // merror conection message
            }
            catch (JsonReaderException jre)
            {
                Console.WriteLine("error " + jre.Message);
            }
        }

        //initialize store location
        private void storeLocation()
        {
            tb_event.Text = " Vaya! Parece que la tienda está cerrada...";
            img_store.Visibility = Visibility.Visible;
            btn_1.Visibility = Visibility.Hidden;
            btn_2.Visibility = Visibility.Hidden;
            btn_3.Visibility = Visibility.Hidden;
            btn_oki.Visibility = Visibility.Visible;
        }

        //initialize start location
        private void startLocation()
        {
            Rolechar c = myModel.getRolechar();
            tb_event.Text = c.TxtIni;
            img_start.Visibility = Visibility.Visible;
            btn_1.Visibility = Visibility.Hidden;
            btn_2.Visibility = Visibility.Hidden;
            btn_3.Visibility = Visibility.Hidden;
            btn_oki.Visibility = Visibility.Visible;
        }

        // initialize end location
        private void endLocation()
        {
            Rolechar c = myModel.getRolechar();
            tb_event.Text = c.TxtFin;
            img_end.Visibility = Visibility.Visible;
            btn_1.Visibility = Visibility.Hidden;
            btn_2.Visibility = Visibility.Hidden;
            btn_3.Visibility = Visibility.Hidden;
            btn_oki.Visibility = Visibility.Visible;
          
        }

        //----------------ANSWERS----------------------------------------------

        /// <summary>
        /// GET petition for list of answers 
        /// </summary>
        private async void getPetitionAnswers(string event_code)
        {                             
                try
                {
                    string URL_getAnswers = Properties.Settings.Default.URL_answers + event_code;
                    Console.WriteLine(URL_getAnswers); // solo test          
                    ResponseResult uResult = await myModel.getPetition(URL_getAnswers);
                    if (uResult.ResultCode == 1)
                    {
                        answers = myModel.convertDataToAnswerOptions(uResult.Data);
                        myModel.setAnswers(answers);
                        // testing solo
                        foreach (Answer_option ans in answers)
                        {
                            if( ans== answers[0])
                        {
                            btn_1.Content = ans.Option_text.ToString();
                        }else if (ans == answers[1])
                        {
                            btn_2.Content = ans.Option_text.ToString();
                        }else if (ans == answers[2])
                        {
                            btn_3.Content = ans.Option_text.ToString();
                        }
                            //Console.WriteLine(ans.ToString());
                        }
                    }
                }
                catch (WebException ex)
                {
                    Console.WriteLine("error " + ex.Message);
                    // merror conection message
                }
                catch (JsonReaderException jre)
                {
                    Console.WriteLine("error " + jre.Message);
                }
            }

        //------------------------response buttons-------------------

        // Button for first response
        private void btn_1_Click(object sender, RoutedEventArgs e)
        {         
            List<Answer_option> ans1 = myModel.getAnswers();                         
            myModel.setAnswer(ans1[0]);
            Console.WriteLine(ans1[0].ToString());
            cons = new Consequence_Window();
            cons.Visibility = Visibility.Visible;
            this.Visibility = Visibility.Hidden;         
        }

        // Button for second response
        private void btn_2_Click(object sender, RoutedEventArgs e)
        {         
            List<Answer_option> ans2 = myModel.getAnswers();           
            myModel.setAnswer(ans2[1]);
            cons = new Consequence_Window();
            cons.Visibility = Visibility.Visible;
            this.Visibility = Visibility.Hidden;      
        }

        // Button for third response
        private void btn_3_Click(object sender, RoutedEventArgs e)
        {
            List<Answer_option> ans3 = myModel.getAnswers();
            myModel.setAnswer(ans3[2]);
            cons = new Consequence_Window();
            cons.Visibility = Visibility.Visible;
            this.Visibility = Visibility.Hidden;        
        }

        //-------------------------------------------------------------

        /// <summary>
        /// initialize background images to hidden
        /// </summary>
        private void initBackgrounds()
        {
            img_start.Visibility = Visibility.Hidden;
            img_store.Visibility = Visibility.Hidden;
            img_park.Visibility = Visibility.Hidden;
            img_bridge.Visibility = Visibility.Hidden;
            img_forest.Visibility = Visibility.Hidden;
            img_square.Visibility = Visibility.Hidden;
            img_end.Visibility = Visibility.Hidden;
        }

        // Button visible for certain events
        private void btn_oki_Click(object sender, RoutedEventArgs e)
        {
            this.Visibility = Visibility.Hidden;         
        }
    }
}
