﻿using Models;
using Models.model;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace AQuarantineAdventure
{
    /// <summary>
    /// Lógica de interacción para MainWindow.xaml 
    /// View-Control type
    /// </summary>
    public partial class MainWindow : Window
    {
        //var
        Model myModel;
        User user;     
        User guest;
        Register_Window rw;

        public MainWindow()
        {
            InitializeComponent();
            myModel = Model.getInstance();
        }

        /// <summary>
        /// when click, validates that the fields are not empty, send the credentials to the model and get a response http back
        /// After getting the response , if its correct, send another http for get the user data and open the next window, 
        /// if isn't  correct, gives a message about the error.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private async void Btn_login_Click(object sender, RoutedEventArgs e)
        {             
            guest = myModel.LoadGuest();
          
            string iduser = tb_user.Text;
            string pass = tb_password.Password;
            user = new User(iduser, pass);

            if (iduser == "" || pass == "")
            {
                MessageBox.Show("Hay campos vacíos!", "Error", MessageBoxButton.OK, MessageBoxImage.Error);
            }
            else if (user.Equals(guest))
            {
                //open next window
                Start_Window sw = new Start_Window();
                myModel.setCurrentUser(guest);
                CleanFields();
                sw.Show();
                this.Hide();
            }
            else if (iduser != "" && pass != "")
            {             
                user = new User(iduser, pass);              
                string URL_login = Properties.Settings.Default.URL_login;
               // Console.WriteLine(URL_login);
                try
                {
                                                       
                  ResponseResult result =  await myModel.deserializeResult(URL_login, user);// transforms User  into a json format
                    if (result.ResultCode == 0)
                    {
                        MessageBox.Show("Error ! El usuario no existe!","Error!", MessageBoxButton.OK, MessageBoxImage.Error);
                        CleanFields();
                    }
                    else if (result.ResultCode == -1)
                    {
                        MessageBox.Show("Error ! La contraseña es incorrecta!", "Error!", MessageBoxButton.OK, MessageBoxImage.Error);
                        CleanFields();
                    }
                    else
                    {
                        string URL_findUser = Properties.Settings.Default.URL_findUser + iduser;
                        //Console.WriteLine(URL_findUser); // solo test

                        // petition for get the user data
                        ResponseResult uResult = await myModel.getPetition(URL_findUser);
                        User uActual = myModel.convertDataToUser(uResult.Data);
                        Start_Window sw = new Start_Window();
                        myModel.setCurrentUser(uActual);
                        CleanFields();
                        sw.Show();
                        this.Hide();
                    }                                           
                }
                catch (WebException ex)
                {
                    Console.WriteLine("error " + ex.Message);
                    // mensaje de error de conexion)
                }catch(JsonReaderException jre){
                    Console.WriteLine("error " + jre.Message);
                }
            }
        }

        /// <summary>
        /// When clicked, opens the windows Register_Window
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Register_click(object sender, MouseButtonEventArgs e)
        {
            rw= new Register_Window();
            rw.Show();
            this.Hide();
        }
       
        /// <summary>
        /// Clean the credential fields
        /// </summary>
        public void CleanFields()
        {
            tb_user.Text = "";
            tb_password.Password = "";
        }

        /// <summary>
        /// window clossing method
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            MessageBoxResult result = MessageBox.Show("Seguro que quieres cerrar el juego?", "Salir", MessageBoxButton.YesNo, MessageBoxImage.Information);

            if (result == MessageBoxResult.Yes)
            {             
                Environment.Exit(0); // close all aplication windows
            }
            else
            {
                e.Cancel = true;
            }
        }  

    }
}
