﻿using Models;
using Models.model;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace AQuarantineAdventure
{

    /// <summary>
    /// Lógica de interacción para Start_Window.xaml
    /// </summary>
    public partial class Start_Window : Window
    {
       
        User uTemp;
        User currentUser;
        Max_score score;   
        Model myModel;       
        MainWindow mw; 
        CharSelect cs;
      
        public Start_Window()
        {
            InitializeComponent();
            myModel = Model.getInstance();
        }

        /// <summary>
        /// initialize the info of the currentUser ( idUser, coins, max_happiness)
        /// </summary>
        public void initUserInfo(object sender, RoutedEventArgs e)
        {         
            try
            {
                currentUser = myModel.getCurrentUser();
                initMaxScore(currentUser);                           
                lb_currentUserId.Content = "Bienvenid@  " + currentUser.IdUser + "!";
                lb_currentCoin.Content = "Monedas: " + currentUser.Coins;
                lb_currentHappy.Content = "Record Felicidad: " + score.Max_happiness;
            }
            catch (NullReferenceException nre)
            {
                Console.WriteLine("error: " + nre.StackTrace);
            }
        }

        /// <summary>
        /// Opens the character selection and then Generate a new game (partida) 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btn_start_Click(object sender, RoutedEventArgs e)
        {
            cs = new CharSelect();
            cs.Visibility = Visibility.Visible;
            this.Visibility = Visibility.Collapsed;
        }

        // return back to the main window
        private void btn_back_Click(object sender, RoutedEventArgs e)
        {
            mw = new MainWindow();
            this.Close();
            mw.Show();
        }

        /// <summary>
        /// gets the max score from  model and sets
        /// </summary>
        private async void initMaxScore(User current)
        {
            score = new Max_score();
            try
            {               
                string URL_maxscore = Properties.Settings.Default.URL_maxscore + current.IdUser;
                Console.WriteLine(URL_maxscore); // solo test          
                ResponseResult uResult = await myModel.getPetition(URL_maxscore);
                if (uResult.ResultCode==0)
                {                  
                    score.Max_happiness = 0;
                    myModel.setMaxScore(score);
                }
                else
                {
                    score = myModel.convertDataToMaxScore(uResult.Data);
                    myModel.setMaxScore(score);
                }              
            }
            catch (WebException ex)
            {
                Console.WriteLine("error " + ex.Message);
                // merror conection message
            }
            catch (JsonReaderException jre)
            {
                Console.WriteLine("error " + jre.Message);
            }           
        }

        /// <summary>
        /// When clicked, asks the user if wants to delete the account, Ok if yes, NO if dont wanna delete the account 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private async void btn_delete_Click(object sender, RoutedEventArgs e)
        {         
            myModel = Model.getInstance();
            if (currentUser.IdUser != "guest")
            {
                MessageBoxResult MBresult = MessageBox.Show("Seguro que quieres eliminar tu cuenta?", "Atención!", MessageBoxButton.YesNo, MessageBoxImage.Question);
                if (MBresult == MessageBoxResult.Yes)
                {
                    mw = new MainWindow();
                    // proces to delete user http petition
                    uTemp = new User(currentUser.IdUser, currentUser.Password);
                    string URL_delete = Properties.Settings.Default.URL_delete;
                    //Console.WriteLine(URL_delete);
                    try
                    {
                        ResponseResult result = await myModel.deserializeResult(URL_delete, uTemp);// transforms User  into a json format
                        if (result.ResultCode == 0)
                        {
                            MessageBox.Show("Error ! Algo sucedió durante el proceso.", "Error!", MessageBoxButton.OK, MessageBoxImage.Error);
                            mw.CleanFields();
                        }
                        else if (result.ResultCode == -1)
                        {
                            MessageBox.Show("Error ! El usuario no existe!", "Error!", MessageBoxButton.OK, MessageBoxImage.Error);
                            mw.CleanFields();
                        }else if(result.ResultCode == 1)
                        {
                            MessageBox.Show("Usuario eliminado correctamente...", "Info", MessageBoxButton.OK, MessageBoxImage.Information);
                            mw.Visibility = Visibility.Visible;
                            this.Visibility= Visibility.Collapsed;                           
                        }
                    }
                    catch (WebException ex)
                    {
                        Console.WriteLine("error " + ex.Message);
                        // connection error
                    }
                    catch (JsonReaderException jre)
                    {
                        Console.WriteLine("error " + jre.Message);
                    }
                }
                else if (MBresult == MessageBoxResult.No)
                {
                    // user not deleted - do nothing
                }
            }
            else
            {
                MessageBox.Show("No puedes eliminar a guest!", "Error", MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        /// <summary>
        /// window clossing method and logOut
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {          
            MessageBoxResult result = MessageBox.Show("Seguro que quieres cerrar sesión?", "Salir", MessageBoxButton.YesNo, MessageBoxImage.Information);           
            if (result == MessageBoxResult.Yes)
            {              
                mw = new MainWindow();             
                mw.Show();
                e.Cancel = false;
            }
            else
            {
                e.Cancel = true;
            }
        }
       
    }
}

   

