﻿using Models.model;
using Models.persistence;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Models.model
{

    public class Model
    {
        //attributes
        private List<Locs> locs = new List<Locs>();
        private List<Answer_option> answers = new List<Answer_option>();
        Answer_option answer;
        private Locs loc;
        private MysqlDAO dao;
        private static Model instanceModel;
        private User currentUser;
        private Max_score maxScore;
        private Rolechar rolechar;
        private Game aGame;
        private Event eve;
        private Consequence conseq;
        Event evento;

        //singleton instance ( not allow duplications)
        public static Model getInstance()
        {
            if (instanceModel == null)
            {
                instanceModel = new Model();
            }
            return instanceModel;
        }
       
        // constructor
        public Model()
        {
            dao = MysqlDAO.getInstance();
            
        }

        // --------------------Common------------------------

        /// <summary>
        /// Gets the  Json response from the httprequest and returns ResponseResult , ONLY GET
        /// </summary>
        /// <param name="Url"></param>
        /// <returns>the deserialized result (object ResponseResult)</returns>
        public async Task<ResponseResult> getPetition(string Url)
        {        
            string getResponse = await dao.GetHttp(Url);
            ResponseResult result = JsonConvert.DeserializeObject<ResponseResult>(getResponse);
            return result;
        }

        // ---------------------METHODS USER--------------------------

        /// <summary>
        ///  Gets the  Json response from the POST request  and returns ResponseResult
        /// </summary>
        /// <param name="Url"> the url for the http request</param>
        /// <param name="u"> the user to send</param>
        /// <returns>the deserialized result (object ResponseResult)</returns>
        public async Task<ResponseResult> deserializeResult(string Url, User u)
        {
            
            string getResponse = await dao.postHttpUser(Url,u);
            ResponseResult result= JsonConvert.DeserializeObject<ResponseResult>(getResponse);
            return result;
        }

        /// <summary>
        ///  Gets the  Json response from the POST request  and returns ResponseResult
        /// </summary>
        /// <param name="Url"> the url for the http request</param>
        /// <param name="u"> the user to actualize</param>
        /// <returns>the deserialized result (object ResponseResult)</returns>
        public async Task<ResponseResult> deserializeActualizeUserResult(string Url, User u)
        {

            string getResponse = await dao.postHttpActualizeUser(Url, u);
            ResponseResult result = JsonConvert.DeserializeObject<ResponseResult>(getResponse);
            return result;
        }

        /// <summary>
        /// Converts data into User
        /// </summary>
        /// <param name="data"> object type data ( the object to deserialize) User</param>
        /// <returns>User currentUser</returns>
        public User convertDataToUser(object data)
        {
            try
            {              
                currentUser = JsonConvert.DeserializeObject<User>(data.ToString());
                Console.WriteLine(currentUser.ToString()); // solo test
            }
            catch (IOException ex)
            {
                Console.WriteLine(ex.Message.ToString());
            }
            return currentUser;
        }

        //-------------------------------- METHODS MAX SCORE---------------------------------
    
        /// <summary>
        ///  POST ---- Gets the  Json response from the request  and returns ResponseResult
        /// </summary>
        /// <param name="Url"> the url for the http request</param>
        /// <param name="ms"> the max score to send (update)</param>
        /// <returns>the deserialized result (object ResponseResult)</returns>
        public async Task<ResponseResult> deserializeResultScore(string Url, Max_score ms)
        {
            
            string getResponse = await dao.postHttpScore(Url, ms);
            ResponseResult result = JsonConvert.DeserializeObject<ResponseResult>(getResponse);
            return result;
        }

        /// <summary>
        /// Converts data to Max_score
        /// </summary>
        /// <param name="data"> object type data ( the object to deserialize) Max_Score</param>
        /// <returns></returns>
        public Max_score convertDataToMaxScore(object data)
        {
            try
            {
                maxScore = JsonConvert.DeserializeObject<Max_score>(data.ToString());
                Console.WriteLine(maxScore.ToString()); // solo test
            }
            catch (IOException ex)
            {
                Console.WriteLine(ex.Message.ToString());
            }
            return maxScore;
        }

        //----------------------METHODS ROLECHAR------------------------------------------------------------

        /// <summary>
        /// Converts data to ROLECHAR
        /// </summary>
        /// <param name="data"> object type data ( the object to deserialize) Max_Score</param>
        /// <returns></returns>
        public Rolechar convertDataToRolechar(object data)
        {
            try
            {
                rolechar = JsonConvert.DeserializeObject<Rolechar>(data.ToString());              
            }
            catch (IOException ex)
            {
                Console.WriteLine(ex.Message.ToString());
            }
            return rolechar;
        }

        //------------------------------METHODS LOCS---------------------------

        /// <summary>
        /// Converts data to LOCS (localization)
        /// </summary>
        /// <param name="data"> object type data ( the object to deserialize) LOCS</param>
        /// <returns></returns>
        public List<Locs> convertDataToLocs(object data)
        {
            try
            {
                locs = JsonConvert.DeserializeObject<List<Locs>>(data.ToString());
            }
            catch (IOException ex)
            {
                Console.WriteLine(ex.Message.ToString());
            }
            return locs;
        }

        //---------------------------METHODS GAME -----------------------------------


        /// <summary>
        ///  Gets the  Json response from the POST request  and returns ResponseResult
        /// </summary>
        /// <param name="Url"> the url for the http request</param>
        /// <param name="game"> the game to send</param>
        /// <returns>the deserialized result (object ResponseResult)</returns>
        public async Task<ResponseResult> deserializeResultGame(string Url, Game game)
        {          
            string getResponse = await dao.postHttpGame(Url, game);
            ResponseResult result = JsonConvert.DeserializeObject<ResponseResult>(getResponse);
            return result;
        }

       

        //------------------------METHODS EVENT--------------------------------------

        /// <summary>
        /// Converts data to event
        /// </summary>
        /// <param name="data"> object type data ( the object to deserialize) Event</param>
        /// <returns></returns>
        public Event convertDataToEvent(object data)
        {
            try
            {
                evento = JsonConvert.DeserializeObject<Event>(data.ToString());
            }
            catch (IOException ex)
            {
                Console.WriteLine(ex.Message.ToString());
            }
            return evento;
        }

        //-----------------------ANSWER OPTION METHODS------------------

        /// <summary>
        /// Converts data to answers 
        /// </summary>
        /// <param name="data"> object type data ( the object to deserialize) Answer_option</param>
        /// <returns></returns>
        public List<Answer_option> convertDataToAnswerOptions(object data)
        {
            try
            {
                answers = JsonConvert.DeserializeObject<List<Answer_option>>(data.ToString());
            }
            catch (IOException ex)
            {
                Console.WriteLine(ex.Message.ToString());
            }
            return answers;
        }


        //-----------------CONSEQUENCE METHODS------------------------------------

        /// <summary>
        /// Converts data to Consequence
        /// </summary>
        /// <param name="data"> object type data ( the object to deserialize) consequence</param>
        /// <returns></returns>
        public Consequence convertDataToConsequence(object data)
        {
            try
            {
                conseq = JsonConvert.DeserializeObject<Consequence>(data.ToString());
            }
            catch (IOException ex)
            {
                Console.WriteLine(ex.Message.ToString());
            }
            return conseq;
        }

        //----------------------------------------------------------------------------

        /// <summary>
        /// loads guest data for local game
        /// </summary>
        public User LoadGuest()
        {
           User uL = new User("guest", "guest");
            return uL;        
        }

        //-----------------------------------GETTERS AND SETTERS------------------
        /// <summary>
        /// setter--- the current loged user from main window
        /// </summary>
        /// <param name="mwUser"></param>
        public void setCurrentUser(User mwUser)
        {
            this.currentUser = mwUser;
        }
        /// <summary>
        /// getter--- the data from the currentUser to model
        /// </summary>
        /// <returns>currentUser</returns>
        public User getCurrentUser()
        {
            return currentUser;
        }

        /// <summary>
        /// setter for max score
        /// </summary>
        /// <param name="score"></param>
        public void setMaxScore(Max_score score)
        {
            this.maxScore = score;
        }

        /// <summary>
        /// getter for max score
        /// </summary>
        /// <returns></returns>
        public Max_score getMaxScore()
        {
            return maxScore;
        }

        /// <summary>
        /// setter for current rolechar
        /// </summary>
        /// <param name="score"></param>
        public void setRolechar(Rolechar role)
        {
            this.rolechar = role;
        }

        /// <summary>
        /// getter for rolechar
        /// </summary>
        /// <returns></returns>
        public Rolechar getRolechar()
        {
            return rolechar;
        }

        /// <summary>
        /// setter for current game
        /// </summary>
        /// <param name="game"></param>
        public void setGame(Game game)
        {
            this.aGame = game;
        }

        /// <summary>
        /// getter for game
        /// </summary>
        /// <returns></returns>
        public Game getGame()
        {
            return aGame;
        }

        /// <summary>
        /// setter for locations list
        /// </summary>
        /// <param name="locs"></param>
        public void setLocs(List<Locs> locs)
        {
            this.locs = locs;
        }
        /// <summary>
        /// getter for locations list
        /// </summary>
        /// <returns></returns>
        public List<Locs> getLocs()
        {
            return locs;
        }

        /// <summary>
        /// setter for loc
        /// </summary>
        /// <param name="loc"></param>
        public void setCurrentLoc(Locs loc)
        {
            this.loc = loc;
        }
        /// <summary>
        /// getter for loc
        /// </summary>
        /// <returns></returns>
        public Locs getCurrentLoc()
        {
            return loc;
        }

        /// <summary>
        /// setter for current event
        /// </summary>
        /// <param name="eve"></param>
        public void setCurrentEvent(Event eve)
        {
            this.eve = eve;
        }
        /// <summary>
        /// getter for current event
        /// </summary>
        /// <returns></returns>
        public Event getCurrentEvent()
        {
            return eve;
        }

        /// <summary>
        /// setter for answers list
        /// </summary>
        /// <param name="answers"></param>
        public void setAnswers(List<Answer_option> answers)
        {
            this.answers = answers;
        }
        /// <summary>
        /// getter for answers list
        /// </summary>
        /// <returns></returns>
        public List<Answer_option> getAnswers()
        {
            return answers;
        }

        /// <summary>
        /// setter for answer
        /// </summary>
        /// <param name="answer"></param>
        public void setAnswer(Answer_option answer)
        {
            this.answer = answer;
        }
        /// <summary>
        /// getter for answer 
        /// </summary>
        /// <returns></returns>
        public Answer_option getAnswer()
        {
            return answer;
        }

        /// <summary>
        /// setter for consequence
        /// </summary>
        /// <param name="consequence"></param>
        public void setConsequence(Consequence consequence)
        {
            this.conseq = consequence;
        }
        /// <summary>
        /// getter for consequence 
        /// </summary>
        /// <returns></returns>
        public Consequence getConsequence()
        {
            return conseq;
        }
    }
}
